/*  ---------------------------------------------------------
	MPI performance test suite
	Test for MPI logical topologies
	Alexander N. Andreyev (alexander@vvv.srcc.msu.su)
	originally developed by Ivan F. Rouzanov
	Laboratory of Parallel Information Technologies, SRCC
	Copyright (c) 1997-1999

    this file: nettest.c (test shell)
    to build the test, attach the following files:
	star.c istar.c chaos.c ichaos.c ring.c ring2.c env.c
----------------------------------------------------------- */

const char last_revision[] = "March/16/1999";

/***************************************************************************
  Include section
***************************************************************************/

#include "mpi-test.h"

/***************************************************************************
  Global data section
***************************************************************************/

int i_am_the_master = 0;
static int msglen_min = iSTART_SIZE, msglen_max = iMAX_SIZE, TIMES = iTIMES;

/***************************************************************************
  Code section
***************************************************************************/


void main (int argc, char *argv[]) {
	int i;
	int iSize;
	int iCode, iNumberOfProc, iMyRank;
	double R[10]; /* rates */
	double t0,t1,TestTime;
	
	MPI_Init (&argc,&argv);

	MPI_Comm_size ( MPI_COMM_WORLD, &iNumberOfProc );
	MPI_Comm_rank ( MPI_COMM_WORLD, &iMyRank );

	t0 = Wtime();
	
	if ( iNumberOfProc < 3)
		finish( "At least 3 nodes required" );

	i_am_the_master = (iMyRank == iROOT );

/* parse command line */
	if(i_am_the_master)	
	{
		printf("\nMPI Network Test\n" );
		printf(" (c) Alexander N. Andreyev, last revision: %s\n",last_revision);
		printf("\nRunning on  %d CPU(s)\n\n", iNumberOfProc );

		for(i = 1; i < argc; i ++)
			switch(argv[i][0]) {
			case 'm': msglen_min = atoi(argv[i]+1); break;
			case 'M': msglen_max = atoi(argv[i]+1); break;
			case 'T': TIMES = atoi(argv[i]+1); break;			
			default: fprintf(stderr,"WARNING: unrecognized option: %s\n",argv[i]);
			break;
		}

		printf("Number of iterations used: %d\n Max. message length: %d\n",TIMES,msglen_max);
		printf("\n====== Transfer rates ===== \n\nSize\tStar\tChaos\tRing\tiStar\tiChaos\tiRing \n\n");


	}

	MPI_Bcast(&msglen_min,1,MPI_INT,iROOT,MPI_COMM_WORLD);
	MPI_Bcast(&msglen_max,1,MPI_INT,iROOT,MPI_COMM_WORLD);
	MPI_Bcast(&TIMES,1,MPI_INT,iROOT,MPI_COMM_WORLD);


	for(iSize = msglen_min; iSize <= msglen_max; iSize *= 2) {

		R[0] = Star   ( iSize, TIMES );     /* Star  topology (blocking)     test. */
		R[1] = Chaos  ( iSize, TIMES );     /* Chaos topology (blocking)     test. */
		R[2] = Ring   ( iSize, TIMES );     /* Ring  topology (blocking)     test. */
		R[3] = iStar  ( iSize, TIMES );     /* Star  topology (non-blocking) test. */
		R[4] = iChaos ( iSize, TIMES );     /* Chaos topology (non-blocking) test. */
		R[5] = Ring2  ( iSize, TIMES );     /* Ring  topology (non-blocking) test. */

		if(i_am_the_master) {
		printf("%d\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\t%.3lf\n",iSize,R[0],R[1],R[2],R[3],R[4],R[5]);
		fflush(stdout);
		}

	}	

	t1 = Wtime();
	TestTime = t1 - t0;
	
	if (i_am_the_master)
		printf ( "\n\nMPI Network Test complete in %lf sec\n",TestTime);

	MPI_Finalize ( );
	exit ( EXIT_SUCCESS );

} /* End of function main ( ) */


/* End of file 'mpi-test.c' */
